// D70D80GPIBControlDlg.cpp 
//
//////////////////////////////////////////////////////////////////
//
//	Suruga Seiki D70/D80 series controllers controlling sample program 
//
//       Note: When this sample program is used it is necessary to add the following 
//             files provided by National Instruments Co., Ltd. to the project:
//
//		- Decl-32.h
//		- gpib-32.obj
//////////////////////////////////////////////////////////////////					

#include "stdafx.h"
#include "D70D80GPIBControl.h"
#include "D70D80GPIBControlDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CD70D80GPIBControlDlg Dialog

CD70D80GPIBControlDlg::CD70D80GPIBControlDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CD70D80GPIBControlDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CD70D80GPIBControlDlg)
	m_iLSpeed = 100;
	m_iPosition = 0;
	m_iRate = 200;
	m_iSpeed = 2000;
	m_strStatus = _T("");
	m_iStep = 100;
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CD70D80GPIBControlDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CD70D80GPIBControlDlg)
	DDX_Control(pDX, IDC_CW, m_pCCWButton);
	DDX_Control(pDX, IDC_CCW, m_pCWButton);
	DDX_Text(pDX, IDC_LSPEED, m_iLSpeed);
	DDV_MinMaxInt(pDX, m_iLSpeed, 0, 9999);
	DDX_Text(pDX, IDC_POSITION, m_iPosition);
	DDX_Text(pDX, IDC_RATE, m_iRate);
	DDV_MinMaxInt(pDX, m_iRate, 0, 9999);
	DDX_Text(pDX, IDC_SPEED, m_iSpeed);
	DDV_MinMaxInt(pDX, m_iSpeed, 1, 240000);
	DDX_Text(pDX, IDC_STATUS, m_strStatus);
	DDX_Text(pDX, IDC_STEP, m_iStep);
	DDV_MinMaxInt(pDX, m_iStep, 1, 9999999);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CD70D80GPIBControlDlg, CDialog)
	//{{AFX_MSG_MAP(CD70D80GPIBControlDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_MODE_CONT, OnModeCont)
	ON_BN_CLICKED(IDC_MODE_ORG, OnModeOrg)
	ON_BN_CLICKED(IDC_MODE_STEP, OnModeStep)
	ON_BN_CLICKED(IDC_SET_POSITION, OnSetPosition)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_CBN_SELCHANGE(IDC_BOARD_NUMBER, OnSelchangeBoardNumber)
	ON_CBN_SELCHANGE(IDC_GPIB_ADDRESS, OnSelchangeGpibAddress)
	ON_CBN_SELCHANGE(IDC_AXIS, OnSelchangeAxis)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_MY_LBUTTON_DOWN,OnDown)
	ON_MESSAGE(WM_MY_LBUTTON_UP,OnUp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////

BOOL CD70D80GPIBControlDlg::OnInitDialog()
{
	// SURUGA >>
	// Sub-classification of the button control
	m_pCWButton.SubclassDlgItem(IDC_CW,this);
	m_pCCWButton.SubclassDlgItem(IDC_CCW,this);
	// << SURUGA

	CDialog::OnInitDialog();


	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}
	SetIcon(m_hIcon, TRUE);	
	SetIcon(m_hIcon, FALSE);
	
	// SURUGA >>
	iUD = 0;		// Specifies that the initialization of GP-IB has not been completed.
	nTimerID=1;		// Initializes the ID of the timer
	bStopFlag = FALSE;
	// Performs initialization processing of each control unit.
	SetEnableControl(FALSE);									// Makes the control units other than the control unit for setting GPIB unusable.
	((CComboBox*)GetDlgItem(IDC_BOARD_NUMBER))->SetCurSel(0);	// Sets the default value (0) of the GPIB board
	((CComboBox*)GetDlgItem(IDC_GPIB_ADDRESS))->SetCurSel(7);	// Sets the default value (7) of the GPIB address of the controller.
	((CComboBox*)GetDlgItem(IDC_AXIS))->SetCurSel(0);			// Sets Axis 1 as the default value of the driving axis.
	((CComboBox*)GetDlgItem(IDC_ORG))->SetCurSel(0);			// Sets ORG1 as the default value of the origin reversion mode.
	((CButton*)GetDlgItem(IDC_MODE_CONT))->SetCheck(1);			// Sets the continuous movement mode as an initial setting of the operation mode.
	((CButton*)GetDlgItem(IDOK))->EnableWindow(TRUE);
	((CButton*)GetDlgItem(IDCANCEL))->EnableWindow(TRUE);

	OnSelchangeGpibAddress();		// Performs initialization processing of GPIB.
	OnSelchangeAxis();				// Performs initialization processing of the movement axis.
	// << SURUGA

	return TRUE;
}

void CD70D80GPIBControlDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

void CD70D80GPIBControlDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); 

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR CD70D80GPIBControlDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CD70D80GPIBControlDlg::OnDown(WPARAM wParam,LPARAM lParam) 
{
	// SURUGA >>
	//
	//	Processing when the movement button (CW or CCW) is pressed
	//	
	//	Examines the movement mode and performs the following processing.
	//      Continuous movement mode : Starts the movement of the stage.
	//	Others: does not do anything. Performs actual movement when the button is released.
	//

	CString strMessage;			// Buffer for transmitting commands
	if (!UpdateData(TRUE)) return;
	if (iUD == 0) return;		// Does not do anything if the device has not been initialized.

	if (GetCheckedRadioButton(IDC_MODE_CONT,IDC_MODE_ORG) == IDC_MODE_CONT) {	// Examines the operation mode selected.
		// Starts the movement when the operation mode of the stage is a continuous movement mode.
		if (wParam == IDC_CW) strMessage.Format("P8000000L%dR%dF%dUG",m_iLSpeed,m_iRate,m_iSpeed);	// Sets the movement command for moving toward CW.
		else				  strMessage.Format("P8000000L%dR%dF%dDG",m_iLSpeed,m_iRate,m_iSpeed);	// Sets the movement command for moving toward CCW
		SendGpibString(GetAxisString()+strMessage);		
		nTimerID = SetTimer(nTimerID,50,NULL);									// Sets the timer interruption for updating the window.
	}
	// << SURUGA
}

void CD70D80GPIBControlDlg::OnUp(WPARAM wParam,LPARAM lParam) 
{
	// SURUGA >>
	//
	//	Processing when the movement button is released
	//
	//	Examines the movement mode and performs the following processing.
	//		Continuous movement mode : stops the stage.
	//		Step movement mode: confirms the movement and stop of the stage.
	//		Origin reverting mode: Sets the memory switch and confirms the movement and stop of the stage.
	//

	CString strMessage;			// Buffer for transmitting the command
	if (iUD == 0) return;		// Does not do anything if the device has not been initialized.

	switch(GetCheckedRadioButton(IDC_MODE_CONT,IDC_MODE_ORG)){		// Examines the movement mode of the stage.
	case IDC_MODE_CONT:												// Stops the stage when the operation mode is a continuous movement mode.
		OnStop();													// Stoppage of the stage
		KillTimer(nTimerID);										// Breaks the timer interruption for updating the window.
		break;

	case IDC_MODE_STEP:												// Starts the movement of the stage if the operation mode is a step movement mode.
		if (!UpdateData(TRUE)) return;
		if (wParam == IDC_CW) strMessage.Format("P%dL%dR%dF%dUG",m_iStep,m_iLSpeed,m_iRate,m_iSpeed);	// Sets the parameter when the movement direction is a CW direction.
		else				  strMessage.Format("P%dL%dR%dF%dDG",m_iStep,m_iLSpeed,m_iRate,m_iSpeed);	// Sets the parameter when the movement direction is a CCW direction.
		SetEnableControl(FALSE);									// Makes all the controls unchangeable.
		((CButton*)GetDlgItem(IDC_STOP))->EnableWindow(TRUE);		// Makes only the STOP button usable.
		ClearSPByteBuffer();										// Clears the contents of the serial poll buffer for confirming the stoppage.
		bStopFlag = FALSE;											// Clears the stop flag.
		SendGpibString(GetAxisString()+strMessage);					// Transmits the movement command.
		CheckStop();												// Confirms the stoppage.
		SetEnableControl(TRUE);										// Makes the control usable.
		break;

	case IDC_MODE_ORG:												// Sets the memory switch and reverts to the origin when the operation mode is an origin reverting mode.
		if (!UpdateData(TRUE)) return;
		strMessage.Format( "1%d",((CComboBox*)GetDlgItem(IDC_ORG))->GetCurSel() );		// Sets the command for changing the origin reverting method.
		SendGpibString("M" + GetAxisString()+strMessage);			// Transmits the command for changing the origin reverting method.
		SetEnableControl(FALSE);									// Makes all the controls unchangeable.
		((CButton*)GetDlgItem(IDC_STOP))->EnableWindow(TRUE);		// Makes only the STOP button usable.
		ClearSPByteBuffer();										// Clears the contents for the serial poll buffer for confirming the stoppage.
		bStopFlag = FALSE;											// Clears the stop flag.
		strMessage.Format( "L%dR%dF%d",m_iLSpeed,m_iRate,m_iSpeed);	// Sets the command for reverting to the origin.
		SendGpibString(GetAxisString()+strMessage);					// Sets the driving parameter for reverting to the origin.
		SendGpibString(GetAxisString()+"O");						// Starts the origin reversion.
		CheckStop();												// Confirms the stoppage.
		SetEnableControl(TRUE);										// Makes the control usable.
		break;
	}
	// << SURUGA
}

void CD70D80GPIBControlDlg::OnModeCont() 
{
	// SURUGA >>
	//
	//	Processing when the continuous movement mode is selected
	//
	((CButton*)GetDlgItem(IDC_STEP))->EnableWindow(FALSE);		// Makes the edit box in the step movement mode unchangeable.
	((CComboBox*)GetDlgItem(IDC_ORG))->EnableWindow(FALSE);		// Makes the ComboBox in the origin reverting mode unchangeable.
	SetButtonCaption(FALSE);									// Sets the text of the driving button.
	// << SURUGA
}

void CD70D80GPIBControlDlg::OnModeOrg() 
{
	// SURUGA >>
	//
	//	Processing when the origin reverting mode is selected
	//
	((CButton*)GetDlgItem(IDC_STEP))->EnableWindow(FALSE);		// Makes the edit box in the step movement mode unchangeable.
	((CComboBox*)GetDlgItem(IDC_ORG))->EnableWindow(TRUE);		// Makes the ComboBox in the origin reverting mode changeable.
	SetButtonCaption(TRUE);										// Sets the text of the driving button.
	// << SURUGA 
}

void CD70D80GPIBControlDlg::OnModeStep() 
{
	// SURUGA >>
	//
	//	Processing when the step movement mode is selected
	//
	((CButton*)GetDlgItem(IDC_STEP))->EnableWindow(TRUE);		// Makes the edit box in the step movement mode changeable.
	((CComboBox*)GetDlgItem(IDC_ORG))->EnableWindow(FALSE);		// Makes the ComboBox in the origin reverting mode unchangeable.
	SetButtonCaption(FALSE);									// Sets the text of the driving button.
	// << SURUGA
}

void CD70D80GPIBControlDlg::OnSetPosition() 
{
	// SURUGA >>
	//
	//	Processing when the position setting button is pressed
	//
	CSetPositionDlg dlgSetPosition;						// Dialog for setting the position
	dlgSetPosition.m_iPosition = GetPositionData();		// Makes the present position an initial value for the dialog.
	if (dlgSetPosition.DoModal()==IDOK)	{		
		m_iPosition = dlgSetPosition.m_iPosition;		// Sets a new position.
		SetPositionData(m_iPosition);
		UpdateData(FALSE);			
	}
	// << SURUGA
}

void CD70D80GPIBControlDlg::OnStop() 
{
	// SURUGA >>
	//
	// Processing when the STOP button is pressed
	//
	if (iUD == 0) return;					// Does not do anything when the GPIB device has not been initialized.
	SendGpibString("E");					// Transmits the command for stopping all the axes.
	while (1) {
		if (GetAxisStatus() != 1) break;	// Confirms the stoppage of the axis.
	}
	bStopFlag=TRUE;					
	m_iPosition = GetPositionData();
	UpdateData(FALSE);
	// << SURUGA
}

void CD70D80GPIBControlDlg::OnTimer(UINT nIDEvent) 
{
	// SURUGA >>
	//
	// Processing when the timer interruption occurs
	//
	// Displays the status and position of the stage in the window.
	//
	if(iUD == 0) return;								// Does not do anything when initialization of the GPIB device has not been completed.
	GetAxisStatus();									// Updates the status for selecting the axis.
	m_iPosition = GetPositionData();					// Obtains the present position.
	UpdateData(FALSE);					

	if (nIDEvent != NULL) CDialog::OnTimer(nIDEvent);	// Carries out a basic class of implement when being called by the timer interruption.
	// CDialog::OnTimer(nIDEvent);

	// << SURUGA
}

void CD70D80GPIBControlDlg::OnOK() 
{
	// SURUGA >>
	//
	// Processing when the OK button is pressed
	//
	if (iUD != 0) ibonl(iUD,0);	

	// << SURUGA

	CDialog::OnOK();
}

void CD70D80GPIBControlDlg::OnCancel() 
{
	// SURUGA >>
	//
	// Processing when the Cancel button is pressed
	//
	if (iUD != 0) ibonl(iUD,0);	

	// << SURUGA

	CDialog::OnCancel();
}

// SURUGA >>

void CD70D80GPIBControlDlg::SetEnableControl(BOOL bFlag){
//
//	Changes over the usable state of each control unit. 
//
	// Sets the OK or Cancel button.
	((CButton*)GetDlgItem(IDOK))->EnableWindow(bFlag);
	((CButton*)GetDlgItem(IDCANCEL))->EnableWindow(bFlag);
	// Sets the driving button.
	((CButton*)GetDlgItem(IDC_CW))->EnableWindow(bFlag);
	((CButton*)GetDlgItem(IDC_CCW))->EnableWindow(bFlag);
	((CButton*)GetDlgItem(IDC_STOP))->EnableWindow(bFlag);
	// Sets the edit box for the driving parameter.
	((CEdit*)GetDlgItem(IDC_LSPEED))->EnableWindow(bFlag);
	((CEdit*)GetDlgItem(IDC_RATE))->EnableWindow(bFlag);
	((CEdit*)GetDlgItem(IDC_SPEED))->EnableWindow(bFlag);
	// Sets the button for changing over the driving mode.
	((CButton*)GetDlgItem(IDC_MODE_CONT))->EnableWindow(bFlag);
	((CButton*)GetDlgItem(IDC_MODE_STEP))->EnableWindow(bFlag);
	((CButton*)GetDlgItem(IDC_MODE_ORG))->EnableWindow(bFlag);
	((CEdit*)GetDlgItem(IDC_STEP))->EnableWindow(FALSE);
	((CComboBox*)GetDlgItem(IDC_ORG))->EnableWindow(FALSE);
	if (bFlag == TRUE) {
		((CEdit*)GetDlgItem(IDC_STEP))    -> EnableWindow( ((CButton*)GetDlgItem(IDC_MODE_STEP))->GetCheck() == 1);
		((CComboBox*)GetDlgItem(IDC_ORG)) -> EnableWindow( ((CButton*)GetDlgItem(IDC_MODE_ORG ))->GetCheck() == 1);
	}
	// Sets the button for changing the position.
	((CButton*)GetDlgItem(IDC_SET_POSITION)) -> EnableWindow(bFlag);
}

void CD70D80GPIBControlDlg::SetButtonCaption(BOOL bOrgFlag){
//
// Determines Valid or Invalid of each driving button according to the driving mode selected.
//
	if (bOrgFlag == TRUE) {
		// Indicates "Origin reversion starts" on both the CW and CCW driving buttons when the ORG mode has been selected.
		((CButton*)GetDlgItem(IDC_CW ))->SetWindowText("Origin");
		((CButton*)GetDlgItem(IDC_CCW))->SetWindowText("Origin");
	}
	else{
		// Indicates the direction of movement on each driving button when the other mode has been selected.
		((CButton*)GetDlgItem(IDC_CW ))->SetWindowText("Move(+)>>>");
		((CButton*)GetDlgItem(IDC_CCW))->SetWindowText("<<<Move(-)");
	}
}

int CD70D80GPIBControlDlg::ConnectGPIB(int iBoardNumber,int iAddress){
//
// GPIB connection processing
//
	if (iUD != 0){
		// Cuts off the connection if GPIB has been connected to the controller.
		ibonl (iUD,0);
	}
	short ln;

	//
	// Initializes the GPIB device.
	//
	ibconfig (iBoardNumber,7,0);						// Makes the automatic serial polling invalid.
	iUD=ibdev(iBoardNumber,iAddress,0,T1s,1,0x140a);	// Initializes the GPIB device.
	if (ibsta & ERR) {									// Judges the GPIB error bit.
		m_strStatus = "GPIB connection error";
		UpdateData(FALSE);
		iUD = 0;										// Specifies that initialization of the device has not been completed.
		return -1;
	}
	Sleep(50);
	
	//
	// Confirms whether the initialized GPIB device exists.
	//
	for (int i=0;i<10;i++){								// Performs confirmation processing ten times.
		ibln(iUD,iAddress,0,&ln);						// Performs connection confirmation.
		if (ibsta & ERR) {								// GPIB error bit judgement
			m_strStatus = "GPIB connection error";
			UpdateData(FALSE);
			ibonl (iUD,0);
			iUD = 0;									// Specifies that initialization of the device has not been completed.
			return -1;
		}
		else if (ln != 0)break;							// Exits from the loop when the connection has been confirmed.
	}
	if (ln == 0) {
		m_strStatus = "The GPIB device does not exist.";
		UpdateData(FALSE);
		ibonl(iUD,0);									// Cuts off the initialized device.
		iUD=0;											// Specifies that initialization of the device has not been completed.
		return -1;
	}
	return 0;
}
	
int CD70D80GPIBControlDlg::GetPositionData(){
//
// Receives the coordinate data from the controller.
//
	CString strPosition;

	if (SendGpibString("?" + GetAxisString()) == FALSE){	
		m_strStatus = "GP-IB character string transmission error";
		UpdateData(FALSE);
		return 0;
	}
	if (ReceiveGpibString(strPosition)== FALSE) {
		m_strStatus = "GP-IB character string receiving error";
		UpdateData(FALSE);
		return 0;
	}
	return atoi(strPosition);
}	

void CD70D80GPIBControlDlg::OnSelchangeBoardNumber() 
{
	OnSelchangeGpibAddress();	
}

void CD70D80GPIBControlDlg::OnSelchangeGpibAddress() 
{
	//
	// Re-connects and confirms GP-IB when the GP-IB address has been changed.
	//
	int iBoardNumber;
	int iAddress;
	iBoardNumber = ((CComboBox*)GetDlgItem(IDC_BOARD_NUMBER))->GetCurSel();
	iAddress = ((CComboBox*)GetDlgItem(IDC_GPIB_ADDRESS))->GetCurSel();
	if (ConnectGPIB(iBoardNumber,iAddress)==0) {
		SetEnableControl(TRUE);
		m_strStatus = "GPIB connection has been completed.";
	}
	else {
		SetEnableControl(FALSE);
		m_strStatus = " The GPIB device can not be found.";
		((CButton*)GetDlgItem(IDOK))->EnableWindow(TRUE);
		((CButton*)GetDlgItem(IDCANCEL))->EnableWindow(TRUE);
	}
	UpdateData(FALSE);

	SendGpibString("#");			// Permits the generation of SRQ.
}

void CD70D80GPIBControlDlg::OnSelchangeAxis() 
{
	SendGpibString(GetAxisString());
	GetAxisStatus();
	m_iPosition = GetPositionData();
	UpdateData(FALSE);
}

int CD70D80GPIBControlDlg::GetAxisStatus(){
// Obtains the status of the selected axis.
// Return value : -1  Cannot communicate on GPIB
//              :  0  Stop state
//              :  1  On moving
//              :  2  CW limit detection
//              :  3  CCW limit detection
//              :  4  Stop at the origin position
//              : 10  Axis selection impossible

	CString strStatus;
	int iStatus = 0;
	if (iUD == 0) return -1;	// Reverts in error when not being connected by GPIB.

	// Obtaining and confirmation of the status

	if (SendGpibString ("%" + GetAxisString())==FALSE){
		m_strStatus = "GP-IB character string transmission error";
		return -1;
	}
	if (ReceiveGpibString(strStatus) == FALSE){
		m_strStatus = "GP-IB character string receiving error";
		return -1;
	}
	if (!(strStatus[1] & 0x80)) {
		m_strStatus = "Axis selection impossible";
		return 10;
	}
	
	if (strStatus[0] & 0x80) {
		m_strStatus = "On moving";
		return 1;
	}
	if ((strStatus[0] & 0x18) == 0x18) {
		m_strStatus = "The axis cannot be selected.";
		return 10;
	}
	if (strStatus[0] & 0x10) {
		m_strStatus = "CW limit detection";
		return 2;
	}
	if (strStatus[0] & 0x08) {
		m_strStatus = "CCW limit detection";
		return 3;
	}
	if (strStatus[0] & 0x40) {
		m_strStatus = "Origin position";
		return 4;
	}
	m_strStatus = "Stop";
	return 0;
}

BOOL CD70D80GPIBControlDlg::SendGpibString(CString strMessage){
//
// Transmits a character string to the GPIB device.
//
	char cCommand[255];
	if (iUD == 0) return FALSE;				// Terminates in error if the GPIB device has not been initialized.
	strMessage += "\r\n";					// Adds the delimiter to the transmission character string.
	strcpy(cCommand,strMessage);			// Converts from variable length character string to fixed length character string.
	int iCount = strMessage.GetLength();	// Obtains the length of character string.
	ibwrt(iUD,cCommand,iCount);				// Transmits a character string to the GPIB device.
	if (ibsta & (ERR | TIMO)) {
		return FALSE;						// Terminates in error when an error or timeout has occurred.
	}
	return TRUE;
}

BOOL CD70D80GPIBControlDlg::ReceiveGpibString(CString& strMessage){
//
// Receives the character string from the GPIB device.
//
	char cReceiveBuf[255];

	if (iUD == 0) return FALSE;				// Terminates in error if the GPIB device has not been initialized.
	ibrd(iUD,cReceiveBuf,255);				// Stores data transmitted from the device in the fixed length character string.
	if (ibsta & (ERR | TIMO)) {
		return FALSE;						// Terminates in error when an error or timeout has occurred.
	}
	strMessage=cReceiveBuf;
	return TRUE;
}

CString CD70D80GPIBControlDlg::GetAxisString(){
//
// Returns the axis-designated character string of the axis which is selected by the ComboBox.
//
	CString strAxis="XYZW";
	return strAxis.Mid( ((CComboBox*)GetDlgItem(IDC_AXIS))->GetCurSel() , 1) + "1";
}

void CD70D80GPIBControlDlg::ClearSPByteBuffer(){
//
// Deletes the serial poll byte which has gathered in the buffer.
//
	char cSPByte;
	if (iUD == 0) return;			// Does not do anything when the device has not been initialized.
	while(1) {
		ibrsp(iUD,&cSPByte);		// Receives the serial poll byte from the device.
		if (cSPByte == 0) break;	// Exits from the loop when the serial poll byte has disappeared.
	}
}

void CD70D80GPIBControlDlg::CheckStop(){
//
// Confirmation processing of the stop of the stage using SRQ.
//
	char cSPByte;
	int iBoardNumber;
	MSG msg;

	SetTimer(nTimerID,200,NULL);	// Sets the timer for updating the window.
					// Note: When the stop of the stage is confirmed using SRQ, 
					//       set the interval of the timer interruption to 200 ms or longer.
					//       If the timer interval is too short, the controller becomes 
					//	 always BUSY because of the transmission and receiving of the 
					// 	 status, so the SRQ flag may not be set.


	iBoardNumber = ((CComboBox*)GetDlgItem(IDC_BOARD_NUMBER))->GetCurSel();	// Receives the board number of the GPIB board selected.
	while (1){
		while (1){
			short sSRQ;
			if( PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) != 0){	// Processes the message in the window.
				TranslateMessage(&msg);	
				DispatchMessage(&msg);
			}

			TestSRQ(iBoardNumber,&sSRQ);	// Confirms whether the SRQ line has been asserted.
			if (sSRQ != 0) break;			// Exits from the loop if the SRQ line has been asserted.
			if (ibsta & ERR) break;			// Exits from the loop if a GPIB error occurs.
			if (bStopFlag) break;			// Exits from the loop if the STOP button has been pressed.
		}
		if (bStopFlag) break;				// Exits from the loop if the STOP button has been pressed.
		ibrsp(iUD,&cSPByte);				// Carries out the serial poll to the controller.
		if (cSPByte != 0x00) break;			// Exits from the loop if the received SRQ is not 0 (zero).
	}

	KillTimer(nTimerID);					// Release the timer interruption.
	SetEnableControl(TRUE);					// Makes the control usable.
	OnTimer(NULL);							// Updates the window.
}

void CD70D80GPIBControlDlg::SetPositionData(int iPositionData){
//
// Changes the position of the stage.
//
	CString strMessage;
	int iabsPositionData;
	iabsPositionData = abs(iPositionData);
	if(iPositionData >= 0)	strMessage.Format("S%d",iPositionData);
	else					strMessage.Format("SL%d",iPositionData);
	SendGpibString(GetAxisString()+strMessage);
}

// << SURUGA
